/**
 * Copyright 2012 Bull S.A.S.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.handler;

import org.apache.felix.gogo.commands.Action;
import org.apache.felix.gogo.commands.Argument;
import org.apache.felix.gogo.commands.Command;
import org.apache.felix.gogo.commands.Option;
import org.apache.felix.ipojo.ComponentInstance;
import org.apache.felix.ipojo.HandlerManager;
import org.apache.felix.ipojo.HandlerManagerFactory;
import org.apache.felix.ipojo.InstanceManager;
import org.apache.felix.ipojo.Pojo;
import org.apache.felix.ipojo.PrimitiveHandler;
import org.apache.felix.ipojo.architecture.HandlerDescription;
import org.apache.felix.ipojo.metadata.Attribute;
import org.apache.felix.ipojo.metadata.Element;
import org.apache.felix.ipojo.util.Logger;
import org.apache.felix.service.command.CommandSession;
import org.mockito.Mock;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.ow2.shelbie.core.IConverterManager;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.lang.reflect.Field;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Properties;

import static java.util.Arrays.asList;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.initMocks;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * A {@code CommandHandlerTestCase} is ...
 *
 * @author Guillaume Sauthier
 */
public class CommandHandlerTestCase {

    private CommandHandler handler;

    private InstanceManager manager;

    @Mock
    private BundleContext bundleContext;

    @Mock
    private Bundle bundle;

    @Mock
    private IConverterManager converter;

    @BeforeMethod
    public void setUp() throws Exception {
        initMocks(this);

        // Mock behaviors
        when(bundleContext.getProperty(Logger.IPOJO_LOG_LEVEL_PROP)).thenReturn("error");
        when(bundleContext.getBundle()).thenReturn(bundle);
        
        when(bundle.loadClass(eq(HelloAction.class.getName()))).thenReturn(HelloAction.class);

        when(converter.convert(eq(String.class), eq("fr"))).thenReturn("fr");
        when(converter.convert(eq(String.class), eq("en"))).thenReturn("en");
        when(converter.convert(eq(String.class), eq("Guillaume"))).thenReturn("Guillaume");

        // Tested component
        handler = new PojoCommandHandler();

        GogoPreparator preparator = new GogoPreparator();
        preparator.bindConverterManager(converter);
        handler.bindActionPreparator(preparator);
        
        // Create a HandlerManager
        HandlerManagerFactory factory = new HandlerManagerFactory(bundleContext,
                createHandlerMetadata(HelloAction.class));
        manager = new HandlerManager(factory, bundleContext, new HandlerManager[] {});

        // Force init the handler with the manager
        Field field = PrimitiveHandler.class.getDeclaredField("m_manager");
        field.setAccessible(true);
        field.set(handler, manager);

        // Init the manager
        manager.configure(createInstanceMetadata(HelloAction.class.getName()), new Properties());
    }

    @Test
    public void testHandlerConfiguration() throws Exception {
        handler.configure(createCommandMetadata(null), createInstanceProperties("1"));
        assertTrue(handler.getCompleters().isEmpty());
    }

    @Test
    public void testHandlerStartupAndExecution() throws Exception {
        handler.configure(createCommandMetadata(null), createInstanceProperties("1"));
        handler.start();
        handler.validate();

        HandlerDescription hd = handler.getDescription();
        assertEquals(hd.getHandlerName(), "org.ow2.shelbie:command");
        Element h = hd.getHandlerInfo();
        Element c = h.getElements("command", null)[0];
        assertEquals(c.getAttribute("scope"), "test");
        assertEquals(c.getAttribute("name"), "hello");
        assertEquals(c.getAttribute("type"), "stateless");
        
        assertEquals(handler.execute(null, asList((Object) "--lang", "fr", "Guillaume")),
                     "Bonjour Guillaume");
        assertEquals(handler.execute(null, asList((Object) "--lang", "en", "Guillaume")),
                     "Hello Guillaume");
    }

    private Dictionary createInstanceProperties(String id) {
        Dictionary d = new Hashtable();
        d.put("instance.name", id);
        return d;
    }

    private Element createCommandMetadata(String type) {
        Element container = new Element("component", null);
        Element c = new Element(CommandHandler.COMMAND_ELEMENT, CommandHandler.NAMESPACE);
        if (type != null) {
            c.addAttribute(new Attribute(CommandHandler.TYPE_ATTRIBUTE, type));
        }
        container.addElement(c);
        return container;
    }

    private Element createInstanceMetadata(String classname) {
        Element i = new Element("instance", null);
        i.addAttribute(new Attribute("classname", classname));
        return i;
    }

    private Element createHandlerMetadata(Class<?> clazz) {
        Element h = new Element("handler", null);
        h.addAttribute(new Attribute("classname", clazz.getName()));
        h.addAttribute(new Attribute("name", "command"));
        h.addAttribute(new Attribute("namespace", "org.ow2.shelbie"));
        Element m = new Element("manipulation", null);
        h.addElement(m);
        return h;
    }

    @Command(scope = "test",
             name = "hello")
    private static class HelloAction implements Action {

        @Option(name = "--lang",
                required = true)
        String lang;

        @Argument
        String target;

        public HelloAction(InstanceManager manager) {}

        public Object execute(CommandSession session) throws Exception {
            return "fr".equals(lang) ? "Bonjour " + target : "Hello " + target;
        }
    }

    private class PojoCommandHandler extends CommandHandler implements Pojo {

        public ComponentInstance getComponentInstance() {
            return CommandHandlerTestCase.this.manager;
        }
    }

}
