/**
 * Copyright 2012 Bull S.A.S.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.handler;

import org.apache.felix.gogo.commands.Action;
import org.apache.felix.gogo.commands.Argument;
import org.apache.felix.gogo.commands.Command;
import org.apache.felix.gogo.commands.Option;
import org.apache.felix.ipojo.InstanceManager;
import org.apache.felix.service.command.CommandSession;
import org.mockito.Mock;
import org.mockito.Spy;
import org.ow2.shelbie.core.IConverterManager;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.Collections;
import java.util.List;

import static java.util.Arrays.asList;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.initMocks;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertSame;

/**
 * A {@code StatelessGogoCommandTestCase} is ...
 *
 * @author Guillaume Sauthier
 */
public class StatelessGogoCommandTestCase {
    
    private StatelessGogoCommand command;
    
    @Mock
    private InstanceManager manager;
    
    @Mock
    private IConverterManager converter;

    @Mock
    private CommandSession session;
    
    @Mock
    private Action action;
    
    @Spy
    private GogoPreparator preparator = new GogoPreparator();

    @BeforeMethod
    public void setUp() throws Exception {
        initMocks(this);
        preparator.bindConverterManager(converter);
        command = new StatelessGogoCommand(manager, preparator);
    }

    @Test
    public void testCreateNewActionReturnsDifferentInstances() throws Exception {
        Action action2 = mock(Action.class);
        Action action3 = mock(Action.class);
        when(manager.createPojoObject()).thenReturn(action, action2, action3);

        // Ensure a different instance is returned each time
        assertSame(command.createNewAction(), action);
        assertSame(command.createNewAction(), action2);
        assertSame(command.createNewAction(), action3);
    }

    @Test
    public void testReleaseAction() throws Exception {
        command.releaseAction(action);
        verify(manager).deletePojoObject(eq(action));
    }

    @Test
    public void testSuccessfulExecution() throws Exception {
        HelloAction action = new HelloAction();
        when(manager.createPojoObject()).thenReturn(action);
        when(converter.convert(eq(String.class), eq("fr"))).thenReturn("fr");
        when(converter.convert(eq(String.class), eq("Guillaume"))).thenReturn("Guillaume");

        assertEquals(command.execute(session, asList((Object) "--lang", "fr", "Guillaume")), "OK");
        assertEquals(action.session, session);

        verify(manager).deletePojoObject(eq(action));
    }

    @Test(expectedExceptions = RuntimeException.class)
    public void testFailedExecution() throws Exception {
        FailedAction action = new FailedAction();
        when(manager.createPojoObject()).thenReturn(action);

        try {
            command.execute(null, Collections.emptyList());
        } finally {
            // Ensure that the action is retired
            verify(manager).deletePojoObject(eq(action));
        }
    }

    @Test
    public void testFailedPreparation() throws Exception {
        HelloAction action = new HelloAction();
        when(manager.createPojoObject()).thenReturn(action);
        doReturn(false).when(preparator).prepare(any(Action.class), eq(session), any(List.class));
        
        assertNull(command.execute(session, asList((Object) "--lang", "fr", "Guillaume")));
        assertNull(action.session);

        verify(manager).deletePojoObject(eq(action));
    }

    @Command(scope = "test",
             name = "hello")
    private static class HelloAction implements Action {

        @Option(name = "--lang",
                required = true)
        String lang;

        @Argument
        String target;

        /**
         * Init to null so that we know if the action has been executed or not.
         */
        CommandSession session = null;

        public Object execute(CommandSession session) throws Exception {
            this.session = session;
            return "OK";
        }
    }

    @Command(scope = "test",
             name = "failed")
    private static class FailedAction implements Action {
        public Object execute(CommandSession session) throws Exception {
            throw new RuntimeException("KO");
        }
    }
}
