/**
 * Copyright 2013 Peergreen SAS
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.registry.supplier.gogo.info;

import org.apache.felix.service.command.CommandProcessor;
import org.apache.felix.service.command.CommandSession;
import org.apache.felix.service.command.Descriptor;
import org.apache.felix.service.command.Parameter;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.ow2.shelbie.core.registry.info.ArgumentInfo;
import org.ow2.shelbie.core.registry.info.CommandInfo;
import org.ow2.shelbie.core.registry.info.OptionInfo;
import org.ow2.shelbie.core.registry.info.ScopeInfo;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.Iterator;
import java.util.List;

import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * User: guillaume
 * Date: 15/02/13
 * Time: 15:36
 */
public class InfoBuilderTestCase {
    @Mock
    private BundleContext bundleContext;
    @Mock
    private ServiceReference reference;

    @BeforeMethod
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void testAnnotationLessCommands() throws Exception {
        when(bundleContext.getService(reference)).thenReturn(new AnnotationLessCommands());
        when(reference.getProperty(CommandProcessor.COMMAND_SCOPE)).thenReturn("test");
        when(reference.getProperty(CommandProcessor.COMMAND_FUNCTION)).thenReturn(new String[] {"execute1", "execute2", "execute3", "execute4"});
        InfoBuilder builder = new InfoBuilder(bundleContext);

        ScopeInfo info = builder.buildScopeInfo(reference);
        assertEquals(info.getName(), "test");

        Iterator<CommandInfo> i = info.getCommands().iterator();
        CommandInfo e_1 = i.next();
        CommandInfo e_2 = i.next();
        CommandInfo e_3 = i.next();
        CommandInfo e_4 = i.next();

        assertEquals(e_1.getName(), "execute1");
        assertTrue(e_1.getArguments().isEmpty());
        assertTrue(e_1.getOptions().isEmpty());

        assertEquals(e_2.getName(), "execute2");
        assertTrue(e_2.getOptions().isEmpty());
        ArgumentInfo e_2_arg_1 = e_2.getArguments().get(0);
        assertEquals(e_2_arg_1.getIndex(), 0);
        assertEquals(e_2_arg_1.getDescription(), null);
        assertEquals(e_2_arg_1.getType(), String.class);
        assertEquals(e_2_arg_1.getDefault(), null);
        assertEquals(e_2_arg_1.isMultiValued(), false);
        assertEquals(e_2_arg_1.isRequired(), true);

        assertEquals(e_3.getName(), "execute3");
        assertTrue(e_3.getOptions().isEmpty());
        ArgumentInfo e_3_arg_1 = e_3.getArguments().get(0);
        assertEquals(e_3_arg_1.getIndex(), 0);
        assertEquals(e_3_arg_1.getDescription(), null);
        assertEquals(e_3_arg_1.getType(), String.class);
        assertEquals(e_3_arg_1.getDefault(), null);
        assertEquals(e_3_arg_1.isMultiValued(), false);
        assertEquals(e_3_arg_1.isRequired(), true);

        assertEquals(e_4.getName(), "execute4");
        assertTrue(e_4.getOptions().isEmpty());
        ArgumentInfo e_4_arg_1 = e_4.getArguments().get(0);
        assertEquals(e_4_arg_1.getIndex(), 0);
        assertEquals(e_4_arg_1.getDescription(), null);
        assertEquals(e_4_arg_1.getType(), List.class);
        assertEquals(e_4_arg_1.getDefault(), null);
        assertEquals(e_4_arg_1.isMultiValued(), true);
        assertEquals(e_4_arg_1.isRequired(), true);
    }

    @Test
    public void testAnnotationCommands() throws Exception {
        when(bundleContext.getService(reference)).thenReturn(new AnnotationCommands());
        when(reference.getProperty(CommandProcessor.COMMAND_SCOPE)).thenReturn("test");
        when(reference.getProperty(CommandProcessor.COMMAND_FUNCTION)).thenReturn(new String[] {"execute1", "execute2", "execute3"});
        InfoBuilder builder = new InfoBuilder(bundleContext);

        ScopeInfo info = builder.buildScopeInfo(reference);
        assertEquals(info.getName(), "test");

        Iterator<CommandInfo> i = info.getCommands().iterator();
        CommandInfo e_1 = i.next();
        CommandInfo e_2 = i.next();
        CommandInfo e_3 = i.next();

        assertEquals(e_1.getName(), "execute1");
        assertTrue(e_1.getArguments().isEmpty());
        assertTrue(e_1.getOptions().isEmpty());
        assertEquals(e_1.getDescription(), "a");

        assertEquals(e_2.getName(), "execute2");
        assertEquals(e_2.getDescription(), "b");
        assertTrue(e_2.getOptions().isEmpty());
        ArgumentInfo e_2_arg_1 = e_2.getArguments().get(0);
        assertEquals(e_2_arg_1.getIndex(), 0);
        assertEquals(e_2_arg_1.getDescription(), "b.0");
        assertEquals(e_2_arg_1.getType(), String.class);
        assertEquals(e_2_arg_1.getDefault(), null);
        assertEquals(e_2_arg_1.isMultiValued(), false);
        assertEquals(e_2_arg_1.isRequired(), true);

        assertEquals(e_3.getName(), "execute3");
        assertEquals(e_3.getDescription(), "c");
        OptionInfo e_3_opt_1 = e_3.getOptions().get(0);
        assertEquals(e_3_opt_1.getDescription(), "c.0");
        assertEquals(e_3_opt_1.isRequired(), false);
        assertEquals(e_3_opt_1.getDefault(), "false");
        assertEquals(e_3_opt_1.getNames().get(0), "-b");
        assertEquals(e_3_opt_1.getNames().get(1), "--bool");

        ArgumentInfo e_3_arg_1 = e_3.getArguments().get(0);
        assertEquals(e_3_arg_1.getIndex(), 0);
        assertEquals(e_3_arg_1.getDescription(), "c.1");
        assertEquals(e_3_arg_1.getType(), List.class);
        assertEquals(e_3_arg_1.getDefault(), null);
        assertEquals(e_3_arg_1.isMultiValued(), true);
        assertEquals(e_3_arg_1.isRequired(), true);
    }

    private static class AnnotationLessCommands {
        public void execute1() {}
        public void execute2(String value) {}
        public void execute3(CommandSession session, String value) {}
        public void execute4(List<String> value) {}
    }

    private static class AnnotationCommands {
        @Descriptor("a")
        public void execute1() {}
        @Descriptor("b")
        public void execute2(@Descriptor("b.0") String value, @Descriptor("b.1") String value2) {}
        @Descriptor("c")
        public void execute3(CommandSession session,
                             @Descriptor("c.0") @Parameter(names = {"-b", "--bool"}, presentValue = Parameter.UNSPECIFIED, absentValue = "false") boolean b,
                             @Descriptor("c.1") List<String> value) {}
    }

}
