/**
 * Copyright 2013 Peergreen SAS
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.testing;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;

import org.apache.felix.gogo.commands.Action;
import org.apache.felix.gogo.commands.Argument;
import org.apache.felix.gogo.commands.Option;
import org.apache.felix.service.command.CommandSession;
import org.fusesource.jansi.Ansi;
import org.testng.annotations.Test;

/**
 * Created with IntelliJ IDEA.
 * User: guillaume
 * Date: 13/02/13
 * Time: 10:40
 * To change this template use File | Settings | File Templates.
 */
public class ActionContainerTestCase {

    @Test
    public void testNoOptionsAction() throws Exception {
        ActionContainer command = new ActionContainer(new NoOptionAction());
        assertTrue((Boolean) command.execute(null));
    }

    @Test
    public void testRequiredOptionAction() throws Exception {
        ActionContainer command = new ActionContainer(new RequiredOptionAction());
        command.option("--boolean", Boolean.TRUE);
        assertTrue((Boolean) command.execute(null));
    }

    @Test(expectedExceptions = IllegalStateException.class)
    public void testMissingRequiredOptionAction() throws Exception {
        ActionContainer command = new ActionContainer(new RequiredOptionAction());
        command.execute(null);
    }

    @Test
    public void testOptionalOptionAction() throws Exception {
        ActionContainer command = new ActionContainer(new OptionalOptionAction());
        command.option("--boolean", Boolean.FALSE);
        assertFalse((Boolean) command.execute(null));
    }

    @Test
    public void testMissingOptionalOptionAction() throws Exception {
        ActionContainer command = new ActionContainer(new OptionalOptionAction());
        assertFalse((Boolean) command.execute(null));
    }

    @Test
    public void testMultipleOptionAction() throws Exception {
        ActionContainer command = new ActionContainer(new MultiOptionAction());
        command.option("--one", "Hello ")
                .option("--two", "world");
        assertEquals(command.execute(null), "Hello world");
    }

    @Test
    public void testAliasedOptionAction() throws Exception {
        ActionContainer command = new ActionContainer(new AliasedOptionAction());
        command.option("-b", Boolean.TRUE);
        assertTrue((Boolean) command.execute(null));

        command = new ActionContainer(new AliasedOptionAction());
        command.option("--bool", Boolean.TRUE);
        assertTrue((Boolean) command.execute(null));

        command = new ActionContainer(new AliasedOptionAction());
        command.option("--boolean", Boolean.TRUE);
        assertTrue((Boolean) command.execute(null));
    }

    @Test
    public void testOptionalArgumentAction() throws Exception {
        ActionContainer command = new ActionContainer(new OptionalArgumentAction());
        command.argument(0, "Hello");
        assertEquals(command.execute(null), "Hello");
    }

    @Test
    public void testMissingOptionalArgumentAction() throws Exception {
        ActionContainer command = new ActionContainer(new OptionalArgumentAction());
        assertEquals(command.execute(null), null);
    }

    @Test
    public void testRequiredArgumentAction() throws Exception {
        ActionContainer command = new ActionContainer(new RequiredArgumentAction());
        command.argument(0, "Hello");
        assertEquals(command.execute(null), "Hello");
    }

    @Test(expectedExceptions = IllegalStateException.class)
    public void testMissingRequiredArgumentAction() throws Exception {
        ActionContainer command = new ActionContainer(new RequiredArgumentAction());
        command.execute(null);
    }

    @Test
    public void testMultiArgumentAction() throws Exception {
        ActionContainer command = new ActionContainer(new MultiArgumentAction());
        command.argument(0, "Hello ");
        command.argument(1, "world");
        assertEquals(command.execute(null), "Hello world");
    }

    @Test
    public void testMultiArgumentAction2() throws Exception {
        ActionContainer command = new ActionContainer(new MultiArgumentAction());
        command.argument(0, "Hello");
        assertEquals(command.execute(null), "Hellonull");
    }


    @Test
    public void testSystemOutAction() throws Exception {
        ActionContainer command = new ActionContainer(new HelloSystemOutAction());
        assertNull(command.execute(null));
        assertEquals(command.getSystemOut(), "Hello");
        assertEquals(command.getSystemErr(), "");
    }

    @Test
    public void testSystemOutColorAction() throws Exception {
        ActionContainer command = new ActionContainer(new HelloSystemOutColorAction());
        assertNull(command.execute(null));
        assertEquals(command.getSystemOut(), "Hello [1;31mworld[m");
        assertEquals(command.getSystemErr(), "");
    }

    @Test
    public void testSystemOutNoAnsiAction() throws Exception {
        ActionContainer command = new ActionContainer(new HelloSystemOutColorAction());
        assertNull(command.execute(null));
        assertEquals(command.getSystemOut(false), "Hello world");
        assertEquals(command.getSystemErr(), "");
    }



    @Test
    public void testSystemOutActionUserDefined() throws Exception {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        PrintStream myStream = new PrintStream(byteArrayOutputStream);

        ActionContainer command = new ActionContainer(new HelloSystemOutAction());
        command.setSystemOutputStream(myStream);
        assertNull(command.execute(null));
        assertEquals(command.getSystemOut(), "Hello");
        assertEquals(command.getSystemErr(), "");

        // also check my stream
        String content = byteArrayOutputStream.toString("UTF-8");
        assertEquals(content, "Hello");
    }

    @Test
    public void testSystemErrAction() throws Exception {
        ActionContainer command = new ActionContainer(new HelloSystemErrAction());
        assertNull(command.execute(null));
        assertEquals(command.getSystemErr(), "MyError");
        assertEquals(command.getSystemOut(), "");
    }

    @Test
    public void testSystemErrActionUserDefined() throws Exception {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        PrintStream myStream = new PrintStream(byteArrayOutputStream);

        ActionContainer command = new ActionContainer(new HelloSystemErrAction());
        command.setSystemErrorStream(myStream);
        assertNull(command.execute(null));
        assertEquals(command.getSystemOut(), "");
        assertEquals(command.getSystemErr(), "MyError");

        // also check my stream
        String content = byteArrayOutputStream.toString("UTF-8");
        assertEquals(content, "MyError");
    }


    @Test
    public void testSystemErrColorAction() throws Exception {
        ActionContainer command = new ActionContainer(new HelloSystemErrColorAction());
        assertNull(command.execute(null));
        assertEquals(command.getSystemErr(), "My [1;31mexception[m");
        assertEquals(command.getSystemOut(), "");
    }

    @Test
    public void testSystemErrNoAnsiAction() throws Exception {
        ActionContainer command = new ActionContainer(new HelloSystemErrColorAction());
        assertNull(command.execute(null));
        assertEquals(command.getSystemErr(false), "My exception");
        assertEquals(command.getSystemOut(), "");
    }




    @Test
    public void testSystemInAction() throws Exception {
        ActionContainer command = new ActionContainer(new SystemInAction());
        command.setSystemIn("myInput");

        // should return true
        assertEquals(command.execute(null), true);

        // should print the system in
        assertEquals(command.getSystemOut(), "myInput");
        assertEquals(command.getSystemErr(), "");

    }

    @Test
    public void testSystemInActionUserDefined() throws Exception {
        InputStream inputStream = new ByteArrayInputStream("florent".getBytes());

        ActionContainer command = new ActionContainer(new SystemInAction());
        command.setSystemInputStream(inputStream);

        // should return true
        assertEquals(command.execute(null), true);

        // should print the system in
        assertEquals(command.getSystemOut(), "florent");
        assertEquals(command.getSystemErr(), "");

    }



    // --------------------------------------------------------

    private static class HelloSystemOutColorAction implements Action {

        public Object execute(CommandSession session) throws Exception {
            Ansi ansi = Ansi.ansi();
            ansi.render("Hello @|bold,red world|@");
            System.out.print(ansi.toString());
            return null;
        }
    }

    private static class HelloSystemOutAction implements Action {

        public Object execute(CommandSession session) throws Exception {
            System.out.print("Hello");
            return null;
        }
    }

    private static class HelloSystemErrAction implements Action {

        public Object execute(CommandSession session) throws Exception {
            System.err.print("MyError");
            return null;
        }
    }

    private static class HelloSystemErrColorAction implements Action {

        public Object execute(CommandSession session) throws Exception {
            Ansi ansi = Ansi.ansi();
            ansi.render("My @|bold,red exception|@");
            System.err.print(ansi.toString());
            return null;
        }
    }

    private static class SystemInAction implements Action {

        public Object execute(CommandSession session) throws Exception {
            try {
                BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
                String input = reader.readLine();
                System.out.print(input);
            } catch (IOException e) {
                return false;
            }
            return true;
        }
    }




    private static class NoOptionAction implements Action {
        public Object execute(CommandSession session) throws Exception {
            return true;
        }
    }

    private static class RequiredOptionAction implements Action {
        @Option(name = "--boolean", required = true)
        private boolean value;
        public Object execute(CommandSession session) throws Exception {
            return value;
        }
    }

    private static class OptionalOptionAction implements Action {
        @Option(name = "--boolean", required = false)
        private boolean value;
        public Object execute(CommandSession session) throws Exception {
            return value;
        }
    }

    private static class AliasedOptionAction implements Action {
        @Option(name = "-b", aliases = {"--boolean", "--bool"}, required = true)
        private boolean value;
        public Object execute(CommandSession session) throws Exception {
            return value;
        }
    }

    private static class MultiOptionAction implements Action {
        @Option(name = "--one", required = true)
        private String one;
        @Option(name = "--two", required = true)
        private String two;
        public Object execute(CommandSession session) throws Exception {
            return one + two;
        }
    }

    private static class OptionalArgumentAction implements Action {
        @Argument
        private String value;
        public Object execute(CommandSession session) throws Exception {
            return value;
        }
    }

    private static class RequiredArgumentAction implements Action {
        @Argument(required = true)
        private String value;
        public Object execute(CommandSession session) throws Exception {
            return value;
        }
    }

    private static class MultiArgumentAction implements Action {
        @Argument(index = 0, required = true)
        private String one;
        @Argument(index = 1, required = false)
        private String two;
        public Object execute(CommandSession session) throws Exception {
            return one + two;
        }
    }
}
